@@ -1,18 +1,23 @@
 /*
  * Copyright 2008-2009 Katholieke Universiteit Leuven
+ * Copyright 2014      Ecole Normale Superieure
  *
- * Use of this software is governed by the GNU LGPLv2.1 license
+ * Use of this software is governed by the MIT license
  *
  * Written by Sven Verdoolaege, K.U.Leuven, Departement
  * Computerwetenschappen, Celestijnenlaan 200A, B-3001 Leuven, Belgium
+ * and Ecole Normale Superieure, 45 rue d'Ulm, 75230 Paris, France
  */
 
 #include <isl_ctx_private.h>
-#include <isl/dim.h>
-#include <isl/seq.h>
+#include <isl_map_private.h>
+#include <isl/space.h>
+#include <isl_seq.h>
 #include <isl_mat_private.h>
-#include "isl_map_private.h"
-#include <isl_dim_private.h>
+#include <isl_vec_private.h>
+#include <isl_space_private.h>
+#include <isl_val_private.h>
+#include <isl/deprecated/mat_int.h>
 
 isl_ctx *isl_mat_get_ctx(__isl_keep isl_mat *mat)
 {
@@ -34,7 +39,7 @@ struct isl_mat *isl_mat_alloc(struct isl_ctx *ctx,
 	if (isl_blk_is_error(mat->block))
 		goto error;
 	mat->row = isl_alloc_array(ctx, isl_int *, n_row);
-	if (!mat->row)
+	if (n_row && !mat->row)
 		goto error;
 
 	for (i = 0; i < n_row; ++i)
@@ -94,7 +99,7 @@ struct isl_mat *isl_mat_extend(struct isl_mat *mat,
 	if (isl_blk_is_error(mat->block))
 		goto error;
 	row = isl_realloc_array(mat->ctx, mat->row, isl_int *, n_row);
-	if (!row)
+	if (n_row && !row)
 		goto error;
 	mat->row = row;
 
@@ -122,7 +127,7 @@ __isl_give isl_mat *isl_mat_sub_alloc6(isl_ctx *ctx, isl_int **row,
 	if (!mat)
 		return NULL;
 	mat->row = isl_alloc_array(ctx, isl_int *, n_row);
-	if (!mat->row)
+	if (n_row && !mat->row)
 		goto error;
 	for (i = 0; i < n_row; ++i)
 		mat->row[i] = row[first_row+i] + first_col;
@@ -204,19 +209,21 @@ struct isl_mat *isl_mat_cow(struct isl_mat *mat)
 	return mat2;
 }
 
-void isl_mat_free(struct isl_mat *mat)
+__isl_null isl_mat *isl_mat_free(__isl_take isl_mat *mat)
 {
 	if (!mat)
-		return;
+		return NULL;
 
 	if (--mat->ref > 0)
-		return;
+		return NULL;
 
 	if (!ISL_F_ISSET(mat, ISL_MAT_BORROWED))
 		isl_blk_free(mat->ctx, mat->block);
 	isl_ctx_deref(mat->ctx);
 	free(mat->row);
 	free(mat);
+
+	return NULL;
 }
 
 int isl_mat_rows(__isl_keep isl_mat *mat)
@@ -243,6 +250,25 @@ int isl_mat_get_element(__isl_keep isl_mat *mat, int row, int col, isl_int *v)
 	return 0;
 }
 
+/* Extract the element at row "row", oolumn "col" of "mat".
+ */
+__isl_give isl_val *isl_mat_get_element_val(__isl_keep isl_mat *mat,
+	int row, int col)
+{
+	isl_ctx *ctx;
+
+	if (!mat)
+		return NULL;
+	ctx = isl_mat_get_ctx(mat);
+	if (row < 0 || row >= mat->n_row)
+		isl_die(ctx, isl_error_invalid, "row out of range",
+			return NULL);
+	if (col < 0 || col >= mat->n_col)
+		isl_die(ctx, isl_error_invalid, "column out of range",
+			return NULL);
+	return isl_val_int_from_isl_int(ctx, mat->row[row][col]);
+}
+
 __isl_give isl_mat *isl_mat_set_element(__isl_take isl_mat *mat,
 	int row, int col, isl_int v)
 {
@@ -281,7 +307,25 @@ error:
 	return NULL;
 }
 
-struct isl_mat *isl_mat_identity(struct isl_ctx *ctx, unsigned n_row)
+/* Replace the element at row "row", column "col" of "mat" by "v".
+ */
+__isl_give isl_mat *isl_mat_set_element_val(__isl_take isl_mat *mat,
+	int row, int col, __isl_take isl_val *v)
+{
+	if (!v)
+		return isl_mat_free(mat);
+	if (!isl_val_is_int(v))
+		isl_die(isl_val_get_ctx(v), isl_error_invalid,
+			"expecting integer value", goto error);
+	mat = isl_mat_set_element(mat, row, col, v->n);
+	isl_val_free(v);
+	return mat;
+error:
+	isl_val_free(v);
+	return isl_mat_free(mat);
+}
+
+__isl_give isl_mat *isl_mat_diag(isl_ctx *ctx, unsigned n_row, isl_int d)
 {
 	int i;
 	struct isl_mat *mat;
@@ -291,13 +335,44 @@ struct isl_mat *isl_mat_identity(struct isl_ctx *ctx, unsigned n_row)
 		return NULL;
 	for (i = 0; i < n_row; ++i) {
 		isl_seq_clr(mat->row[i], i);
-		isl_int_set_si(mat->row[i][i], 1);
+		isl_int_set(mat->row[i][i], d);
 		isl_seq_clr(mat->row[i]+i+1, n_row-(i+1));
 	}
 
 	return mat;
 }
 
+__isl_give isl_mat *isl_mat_identity(isl_ctx *ctx, unsigned n_row)
+{
+	if (!ctx)
+		return NULL;
+	return isl_mat_diag(ctx, n_row, ctx->one);
+}
+
+/* Is "mat" a (possibly scaled) identity matrix?
+ */
+int isl_mat_is_scaled_identity(__isl_keep isl_mat *mat)
+{
+	int i;
+
+	if (!mat)
+		return -1;
+	if (mat->n_row != mat->n_col)
+		return 0;
+
+	for (i = 0; i < mat->n_row; ++i) {
+		if (isl_seq_first_non_zero(mat->row[i], i) != -1)
+			return 0;
+		if (isl_int_ne(mat->row[0][0], mat->row[i][i]))
+			return 0;
+		if (isl_seq_first_non_zero(mat->row[i] + i + 1,
+					    mat->n_col - (i + 1)) != -1)
+			return 0;
+	}
+
+	return 1;
+}
+
 struct isl_vec *isl_mat_vec_product(struct isl_mat *mat, struct isl_vec *vec)
 {
 	int i;
@@ -483,7 +558,7 @@ static void oppose(struct isl_mat *M, struct isl_mat **U,
  * with U and Q unimodular matrices and H a matrix in column echelon form
  * such that on each echelon row the entries in the non-echelon column
  * are non-negative (if neg == 0) or non-positive (if neg == 1)
- * and stricly smaller (in absolute value) than the entries in the echelon
+ * and strictly smaller (in absolute value) than the entries in the echelon
  * column.
  * If U or Q are NULL, then these matrices are not computed.
  */
@@ -902,6 +977,9 @@ struct isl_mat *isl_mat_transpose(struct isl_mat *mat)
 	struct isl_mat *transpose = NULL;
 	int i, j;
 
+	if (!mat)
+		return NULL;
+
 	if (mat->n_col == mat->n_row) {
 		mat = isl_mat_cow(mat);
 		if (!mat)
@@ -957,7 +1035,13 @@ struct isl_mat *isl_mat_swap_rows(struct isl_mat *mat, unsigned i, unsigned j)
 	return mat;
 }
 
-struct isl_mat *isl_mat_product(struct isl_mat *left, struct isl_mat *right)
+/* Calculate the product of two matrices.
+ *
+ * This function is optimized for operand matrices that contain many zeros and
+ * skips multiplications where we know one of the operands is zero.
+ */
+__isl_give isl_mat *isl_mat_product(__isl_take isl_mat *left,
+	__isl_take isl_mat *right)
 {
 	int i, j, k;
 	struct isl_mat *prod;
@@ -976,10 +1060,13 @@ struct isl_mat *isl_mat_product(struct isl_mat *left, struct isl_mat *right)
 		return prod;
 	}
 	for (i = 0; i < prod->n_row; ++i) {
-		for (j = 0; j < prod->n_col; ++j) {
+		for (j = 0; j < prod->n_col; ++j)
 			isl_int_mul(prod->row[i][j],
 				    left->row[i][0], right->row[0][j]);
-			for (k = 1; k < left->n_col; ++k)
+		for (k = 1; k < left->n_col; ++k) {
+			if (isl_int_is_zero(left->row[i][k]))
+				continue;
+			for (j = 0; j < prod->n_col; ++j)
 				isl_int_addmul(prod->row[i][j],
 					    left->row[i][k], right->row[k][j]);
 		}
@@ -1036,7 +1123,7 @@ static int preimage(struct isl_ctx *ctx, isl_int **q, unsigned n,
  * M the matrix mat.
  *
  * If there are fewer variables x' then there are x, then we perform
- * the transformation in place, which that, in principle,
+ * the transformation in place, which means that, in principle,
  * this frees up some extra variables as the number
  * of columns remains constant, but we would have to extend
  * the div array too as the number of rows in this array is assumed
@@ -1064,7 +1151,7 @@ struct isl_basic_set *isl_basic_set_preimage(struct isl_basic_set *bset,
 		if (!bset)
 			goto error;
 	} else if (mat->n_col < mat->n_row) {
-		bset->dim = isl_dim_cow(bset->dim);
+		bset->dim = isl_space_cow(bset->dim);
 		if (!bset->dim)
 			goto error;
 		bset->dim->n_out -= mat->n_row - mat->n_col;
@@ -1100,14 +1187,12 @@ error2:
 
 struct isl_set *isl_set_preimage(struct isl_set *set, struct isl_mat *mat)
 {
-	struct isl_ctx *ctx;
 	int i;
 
 	set = isl_set_cow(set);
 	if (!set)
 		return NULL;
 
-	ctx = set->ctx;
 	for (i = 0; i < set->n; ++i) {
 		set->p[i] = isl_basic_set_preimage(set->p[i],
 						    isl_mat_copy(mat));
@@ -1115,7 +1200,7 @@ struct isl_set *isl_set_preimage(struct isl_set *set, struct isl_mat *mat)
 			goto error;
 	}
 	if (mat->n_col != mat->n_row) {
-		set->dim = isl_dim_cow(set->dim);
+		set->dim = isl_space_cow(set->dim);
 		if (!set->dim)
 			goto error;
 		set->dim->n_out += mat->n_col;
@@ -1235,6 +1320,9 @@ struct isl_mat *isl_mat_drop_cols(struct isl_mat *mat, unsigned col, unsigned n)
 {
 	int r;
 
+	if (n == 0)
+		return mat;
+
 	mat = isl_mat_cow(mat);
 	if (!mat)
 		return NULL;
@@ -1499,6 +1587,26 @@ error:
 	return NULL;
 }
 
+/* Return a copy of row "row" of "mat" as an isl_vec.
+ */
+__isl_give isl_vec *isl_mat_get_row(__isl_keep isl_mat *mat, unsigned row)
+{
+	isl_vec *v;
+
+	if (!mat)
+		return NULL;
+	if (row >= mat->n_row)
+		isl_die(mat->ctx, isl_error_invalid, "row out of range",
+			return NULL);
+
+	v = isl_vec_alloc(isl_mat_get_ctx(mat), mat->n_col);
+	if (!v)
+		return NULL;
+	isl_seq_cpy(v->el, mat->row[row], mat->n_col);
+
+	return v;
+}
+
 __isl_give isl_mat *isl_mat_vec_concat(__isl_take isl_mat *top,
 	__isl_take isl_vec *bot)
 {
@@ -1582,6 +1690,21 @@ __isl_give isl_mat *isl_mat_scale_down(__isl_take isl_mat *mat, isl_int m)
 	return mat;
 }
 
+__isl_give isl_mat *isl_mat_scale_down_row(__isl_take isl_mat *mat, int row,
+	isl_int m)
+{
+	if (isl_int_is_one(m))
+		return mat;
+
+	mat = isl_mat_cow(mat);
+	if (!mat)
+		return NULL;
+
+	isl_seq_scale_down(mat->row[row], mat->row[row], m, mat->n_col);
+
+	return mat;
+}
+
 __isl_give isl_mat *isl_mat_normalize(__isl_take isl_mat *mat)
 {
 	isl_int gcd;
